// Generated by gir (https://github.com/gtk-rs/gir @ c93f8f31b6a5)
// from  (@ 3879ff8e0201)
// from gir-files (@ 01ea5f1a07cd)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GlyCicp", sizeof(GlyCicp), alignof(GlyCicp));
    printf("%s;%zu;%zu\n", "GlyCreatorClass", sizeof(GlyCreatorClass), alignof(GlyCreatorClass));
    printf("%s;%zu;%zu\n", "GlyEncodedImageClass", sizeof(GlyEncodedImageClass), alignof(GlyEncodedImageClass));
    printf("%s;%zu;%zu\n", "GlyFrameClass", sizeof(GlyFrameClass), alignof(GlyFrameClass));
    printf("%s;%zu;%zu\n", "GlyFrameRequestClass", sizeof(GlyFrameRequestClass), alignof(GlyFrameRequestClass));
    printf("%s;%zu;%zu\n", "GlyImageClass", sizeof(GlyImageClass), alignof(GlyImageClass));
    printf("%s;%zu;%zu\n", "GlyLoaderClass", sizeof(GlyLoaderClass), alignof(GlyLoaderClass));
    printf("%s;%zu;%zu\n", "GlyLoaderError", sizeof(GlyLoaderError), alignof(GlyLoaderError));
    printf("%s;%zu;%zu\n", "GlyMemoryFormat", sizeof(GlyMemoryFormat), alignof(GlyMemoryFormat));
    printf("%s;%zu;%zu\n", "GlyMemoryFormatSelection", sizeof(GlyMemoryFormatSelection), alignof(GlyMemoryFormatSelection));
    printf("%s;%zu;%zu\n", "GlyNewFrameClass", sizeof(GlyNewFrameClass), alignof(GlyNewFrameClass));
    printf("%s;%zu;%zu\n", "GlySandboxSelector", sizeof(GlySandboxSelector), alignof(GlySandboxSelector));
    return 0;
}
