// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files
// DO NOT EDIT

use glib::translate::*;

use crate::ffi;

glib::wrapper! {
    #[doc(alias = "GlyNewFrame")]
    pub struct NewFrame(Object<ffi::GlyNewFrame, ffi::GlyNewFrameClass>);

    match fn {
        type_ => || ffi::gly_new_frame_get_type(),
    }
}

impl NewFrame {
    #[doc(alias = "gly_new_frame_set_color_icc_profile")]
    pub fn set_color_icc_profile(&self, icc_profile: &glib::Bytes) -> bool {
        unsafe {
            from_glib(ffi::gly_new_frame_set_color_icc_profile(
                self.to_glib_none().0,
                icc_profile.to_glib_none().0,
            ))
        }
    }
}

unsafe impl Send for NewFrame {}
unsafe impl Sync for NewFrame {}
